@@ -18,6 +18,7 @@ use crate::handshake::handshake_message_server_key_exchange::*;
 use crate::handshake::handshake_random::*;
 use crate::signature_hash_algorithm::*;
 
+use crate::extension::renegotiation_info::ExtensionRenegotiationInfo;
 use rand::Rng;
 use std::time::SystemTime;
 use util::conn::conn_pipe::*;
@@ -548,7 +549,7 @@ async fn test_psk() -> Result<()> {
         let (ca, cb) = pipe();
         tokio::spawn(async move {
             let conf = Config {
-                psk: Some(psk_callback_client),
+                psk: Some(Arc::new(psk_callback_client)),
                 psk_identity_hint: Some(client_identity.to_vec()),
                 cipher_suites: vec![CipherSuiteId::Tls_Psk_With_Aes_128_Ccm_8],
                 ..Default::default()
@@ -559,7 +560,7 @@ async fn test_psk() -> Result<()> {
         });
 
         let config = Config {
-            psk: Some(psk_callback_server),
+            psk: Some(Arc::new(psk_callback_server)),
             psk_identity_hint: server_identity,
             cipher_suites: vec![CipherSuiteId::Tls_Psk_With_Aes_128_Ccm_8],
             ..Default::default()
@@ -614,7 +615,7 @@ async fn test_psk_hint_fail() -> Result<()> {
     let (ca, cb) = pipe();
     tokio::spawn(async move {
         let conf = Config {
-            psk: Some(psk_callback_hint_fail),
+            psk: Some(Arc::new(psk_callback_hint_fail)),
             psk_identity_hint: Some(vec![]),
             cipher_suites: vec![CipherSuiteId::Tls_Psk_With_Aes_128_Ccm_8],
             ..Default::default()
@@ -625,7 +626,7 @@ async fn test_psk_hint_fail() -> Result<()> {
     });
 
     let config = Config {
-        psk: Some(psk_callback_hint_fail),
+        psk: Some(Arc::new(psk_callback_hint_fail)),
         psk_identity_hint: Some(vec![]),
         cipher_suites: vec![CipherSuiteId::Tls_Psk_With_Aes_128_Ccm_8],
         ..Default::default()
@@ -707,6 +708,8 @@ async fn test_client_timeout() -> Result<()> {
     Ok(())
 }
 
+//use std::io::Write;
+
 #[tokio::test]
 async fn test_srtp_configuration() -> Result<()> {
     /*env_logger::Builder::new()
@@ -724,7 +727,14 @@ async fn test_srtp_configuration() -> Result<()> {
     .filter(None, LevelFilter::Trace)
     .init();*/
 
-    let tests = vec![
+    let tests: Vec<(
+        &str,
+        Vec<SrtpProtectionProfile>,
+        Vec<SrtpProtectionProfile>,
+        SrtpProtectionProfile,
+        Option<Error>,
+        Option<Error>,
+    )> = vec![
         (
             "No SRTP in use",
             vec![],
@@ -822,14 +832,17 @@ async fn test_srtp_configuration() -> Result<()> {
                 assert!(false, "{} expected error, but got ok", name);
             }
         } else {
-            if let Ok(server) = result {
-                let actual_server_srtp = server.selected_srtpprotection_profile();
-                assert_eq!(actual_server_srtp, expected_profile,
-                           "test_srtp_configuration: Server SRTPProtectionProfile Mismatch '{}': expected({:?}) actual({:?})",
-                           name, expected_profile, actual_server_srtp);
-            } else {
-                assert!(false, "{} expected no error", name);
-            }
+            match result {
+                Ok(server) => {
+                    let actual_server_srtp = server.selected_srtpprotection_profile();
+                    assert_eq!(actual_server_srtp, expected_profile,
+                               "test_srtp_configuration: Server SRTPProtectionProfile Mismatch '{}': expected({:?}) actual({:?})",
+                               name, expected_profile, actual_server_srtp);
+                }
+                Err(err) => {
+                    assert!(false, "{} expected no error: {}", name, err);
+                }
+            };
         }
 
         let client_result = client_res_rx.recv().await;
@@ -1428,7 +1441,7 @@ async fn test_server_certificate() -> Result<()> {
             Config {
                 certificates: vec![cert.clone()],
                 client_auth: ClientAuthType::RequireAnyClientCert,
-                verify_peer_certificate: Some(fn_not_expected_chain),
+                verify_peer_certificate: Some(Arc::new(fn_not_expected_chain)),
                 ..Default::default()
             },
             false,
@@ -1447,7 +1460,7 @@ async fn test_server_certificate() -> Result<()> {
                 client_cert_verifier: Some(rustls::AllowAnyAuthenticatedClient::new(
                     roots_cas.clone(),
                 )),
-                verify_peer_certificate: Some(fn_expected_chain),
+                verify_peer_certificate: Some(Arc::new(fn_expected_chain)),
                 ..Default::default()
             },
             false,
@@ -1457,7 +1470,7 @@ async fn test_server_certificate() -> Result<()> {
             Config {
                 roots_cas: roots_cas.clone(),
                 server_name: server_name.clone(),
-                verify_peer_certificate: Some(fn_wrong_cert),
+                verify_peer_certificate: Some(Arc::new(fn_wrong_cert)),
                 ..Default::default()
             },
             Config {
@@ -1772,7 +1785,11 @@ async fn test_psk_configuration() -> Result<()> {
         let (ca, cb) = pipe();
         tokio::spawn(async move {
             let conf = Config {
-                psk: if client_psk { Some(psk_callback) } else { None },
+                psk: if client_psk {
+                    Some(Arc::new(psk_callback))
+                } else {
+                    None
+                },
                 psk_identity_hint: client_psk_identity,
                 ..Default::default()
             };
@@ -1782,7 +1799,11 @@ async fn test_psk_configuration() -> Result<()> {
         });
 
         let config = Config {
-            psk: if server_psk { Some(psk_callback) } else { None },
+            psk: if server_psk {
+                Some(Arc::new(psk_callback))
+            } else {
+                None
+            },
             psk_identity_hint: server_psk_identity,
             ..Default::default()
         };
@@ -2351,7 +2372,6 @@ async fn test_multiple_hello_verify_request() -> Result<()> {
         packets.push(packet);
     }
 
-    use util::Conn;
     let (ca, cb) = pipe();
 
     tokio::spawn(async move {
@@ -2395,3 +2415,136 @@ async fn test_multiple_hello_verify_request() -> Result<()> {
 
     Ok(())
 }
+
+async fn send_client_hello(
+    cookie: Vec<u8>,
+    ca: &Arc<dyn Conn + Send + Sync>,
+    sequence_number: u64,
+    send_renegotiation_info: bool,
+) -> Result<()> {
+    let mut extensions = vec![];
+    if send_renegotiation_info {
+        extensions.push(Extension::RenegotiationInfo(ExtensionRenegotiationInfo {
+            renegotiated_connection: 0,
+        }));
+    }
+
+    let mut h = Handshake::new(HandshakeMessage::ClientHello(HandshakeMessageClientHello {
+        version: PROTOCOL_VERSION1_2,
+        random: HandshakeRandom::default(),
+        cookie,
+
+        cipher_suites: vec![CipherSuiteId::Tls_Ecdhe_Ecdsa_With_Aes_128_Gcm_Sha256],
+        compression_methods: default_compression_methods(),
+        extensions,
+    }));
+    h.handshake_header.message_sequence = sequence_number as u16;
+
+    let mut record = RecordLayer::new(PROTOCOL_VERSION1_2, 0, Content::Handshake(h));
+    record.record_layer_header.sequence_number = sequence_number;
+
+    let mut packet = vec![];
+    {
+        let mut writer = BufWriter::<&mut Vec<u8>>::new(packet.as_mut());
+        record.marshal(&mut writer)?;
+    }
+
+    ca.send(&packet).await?;
+
+    Ok(())
+}
+
+// Assert that a DTLS Server always responds with RenegotiationInfo if
+// a ClientHello contained that extension or not
+#[tokio::test]
+async fn test_renegotation_info() -> Result<()> {
+    let mut resp = vec![0u8; 1024];
+
+    let tests = vec![
+        ("Include RenegotiationInfo", true),
+        ("No RenegotiationInfo", false),
+    ];
+
+    for (name, send_renegotiation_info) in tests {
+        let (ca, cb) = pipe();
+
+        tokio::spawn(async move {
+            let conf = Config::default();
+            let _ = tokio::time::timeout(
+                Duration::from_millis(100),
+                create_test_server(Arc::new(cb), conf, true),
+            )
+            .await;
+        });
+
+        tokio::time::sleep(Duration::from_millis(5)).await;
+
+        let ca: Arc<dyn Conn + Send + Sync> = Arc::new(ca);
+        send_client_hello(vec![], &ca, 0, send_renegotiation_info).await?;
+
+        let n = ca.recv(&mut resp).await?;
+        let mut reader = BufReader::new(&resp[..n]);
+        let record = RecordLayer::unmarshal(&mut reader)?;
+
+        let hello_verify_request = match record.content {
+            Content::Handshake(h) => match h.handshake_message {
+                HandshakeMessage::HelloVerifyRequest(hvr) => hvr,
+                _ => {
+                    assert!(false, "unexpected handshake message");
+                    return Ok(());
+                }
+            },
+            _ => {
+                assert!(false, "unexpected content");
+                return Ok(());
+            }
+        };
+
+        send_client_hello(
+            hello_verify_request.cookie.clone(),
+            &ca,
+            1,
+            send_renegotiation_info,
+        )
+        .await?;
+        let n = ca.recv(&mut resp).await?;
+        let messages = unpack_datagram(&resp[..n])?;
+
+        let mut reader = BufReader::new(&messages[0][..]);
+        let record = RecordLayer::unmarshal(&mut reader)?;
+
+        let server_hello = match record.content {
+            Content::Handshake(h) => match h.handshake_message {
+                HandshakeMessage::ServerHello(sh) => sh,
+                _ => {
+                    assert!(false, "unexpected handshake message");
+                    return Ok(());
+                }
+            },
+            _ => {
+                assert!(false, "unexpected content");
+                return Ok(());
+            }
+        };
+
+        let mut got_negotation_info = false;
+        for v in &server_hello.extensions {
+            match v {
+                Extension::RenegotiationInfo(_) => {
+                    got_negotation_info = true;
+                }
+                _ => {}
+            }
+        }
+
+        assert!(
+            got_negotation_info,
+            "{}: Received ServerHello without RenegotiationInfo",
+            name
+        );
+
+        ca.close().await?;
+    }
+
+    Ok(())
+}
